/**
 * @file  			server.js
 * @description   	开发系统后台服务
 * @author 			xinguo.wang
 * @createDate 		2018/06/02
 * @version   		1.0
 */

// 客户端/服务端枚举
var kmCSTypeEnum = {
	KM_COMM_NULL: null,
	KM_COMM_SERVER: 0,
	KM_COMM_CLIENT: 1
}
// 协议类名枚举
var kmProtocolTypeEnum = {
	KM_COMM_HTTP: "KMHttp",
	KM_COMM_HTTPS: "KMHttps",
	KM_COMM_WS: "KMWs",
	KM_COMM_WSS: "KMWss",
}

// GraphicModelManager.json配置文件中的对象名称
var IDS_MODE_COMPOSITE_TIMPSTAPE                                      = '组合图形模型时间戳'; 
var IDS_MODE_ASSOCIATED_TIMPSTAPE                                     = '关联图形模型时间戳'; 
var IDS_MODE_BASIC_TIMPSTAPE                                          = '基本图形模型时间戳';
var IDS_MODE_COMPOSITE                                            	  = '组合图形模型库'; 
var IDS_MODE_ASSOCIATED                                               = '关联图形模型库'; 
var IDS_MODE_BASIC                                                    = '基本图形模型库'; 
global.deployExsit = true;
var fs = require('fs');
var options = {
	key: fs.readFileSync('./keys/private.key'),
	ca:  fs.readFileSync('./keys/csr.pem'),
	cert: fs.readFileSync('./keys/file.crt')
};

var communication = require('./Communication/km_communication.js');
var KMCommunicationFactory = communication.KMCommunicationFactory;
var factory = new KMCommunicationFactory();
var KMHttp = communication.KMHttp;
var KMHttps = communication.KMHttps;
var KMWs = communication.KMWs;
var KMWss = communication.KMWss;

// 创建文件操作对象
var FileOperate = require('./Frame/FileOperate/km_fileoperate.js');
var KMFileReadWrite = FileOperate.KMFileReadWrite;
var fileOperateObj = new KMFileReadWrite();
// 获取服务端信息
//var config = require('./Config/km_serverparam.json');

// add by qiankun.xie  2019.07.15
// 创建文件操作对象
var FileEncrypt = require('./Frame/FileEncrypt/km_fileencrypt.js');
var KMFileEncrypt = FileEncrypt.KMFileEncrypt;
var fileEncryptObj = new KMFileEncrypt();
// end by qiankun.xie

// add by qiankun.xie  2019.06.28
// 创建数据库操作对象
var RedisDB = require('./Frame/FileEncrypt/km_redisdb.js');
var KMRedisDB = RedisDB.KMRedisDB;
var redisDBObj = new KMRedisDB();
// redisDBObj.createClient("127.0.0.1", 6379);
// end by qiankun.xie
//Modify BUG384 by fengtao 2019-3-20
var jsonStr = fileOperateObj.syncReadFile('./Config/km_serverparam.json', "utf-8");
if(jsonStr.indexOf('\\') !== -1){
	jsonStr = jsonStr.replace(/\\/g,'/');
}
var config = JSON.parse(jsonStr);
//Modify BUG384 by fengtao 2019-3-20

// jingyu.liu{url用户管理功能添加}
var loginManger = require('./Frame//LoginInfoManager/km_login_Info_manger.js');
var loginMng = new loginManger.KMLoginInfoManager();
//WS 管理
var wsLoginMng = new loginManger.KMWSLoginManager();
var userIdManager = new loginManger.KMUserIDManager();

// 获取路由
var router = require('./Frame/Router/km_router.js');
var KMUrlRouter = router.KMUrlRouter;
var urlRouterObj = new KMUrlRouter(loginMng, userIdManager);
var currentProName = '';

var url = require('url');
var qs = require('querystring');
var iconv = require('iconv-lite');

// add for web gis.
var pathModule = require('path');
var fs = require('fs');
var mime = require('mime-types');
var TileFileAsync = require('./tileAsync/tileFileAsync.js').TileFileAsync;
// add for web gis end.

// 引用KM1.0中自行修改整合过的cryptico.js库
api = require("./Frame/Encrypt/api.js");
var num = 0;
var PassPhrase = "This is the server key.";
var Bits = 512;
// 初始化cryptico.js库,使用之前一定要调用
api.Init();
// 生成密钥对
ServerRSAkey = api.generateRSAKey(PassPhrase, Bits);
// 提取出公钥字符串
ServerPublicKeyString = api.publicKeyString(ServerRSAkey); 

// added by yu.sun for 页面组权限秘钥
var modulekey = null, modulekeyString = null;
var KMEncryptionFile = require("./Frame/Encrypt/km_encryption.js");
var KMEncryption = new KMEncryptionFile.KMEncryption()
// end

// xinguo.wang{演示模式功能添加}
var demomode = require('./Frame/DemoMode/km_demomode.js');
var KMDemoModeManager = demomode.KMDemoModeManager;
var demomng = new KMDemoModeManager();

// added by yun.sun 权限配置 2019 -07 -12
var wsAuthortionMap = new Map()
//end

/**
 * @class     Server
 * @description  构造函数
 * @return       {undefined}
 * @example
 */
function Server() {
	// 服务端参数信息
	this.config = config;
	// 服务类型管理数组
	this.typeMng = new Object();
}

/**
 * @method        	getSSLSync
 * @description     获取SSL证书
 * @returns         {Object}	options		返回证书对象信息
 */
Server.prototype.getSSLSync = function () {
	var options = {};
	if (config.SSL) {
		var keyExists = fileOperateObj.existsSync(config.SSL.key);
		var certExists = fileOperateObj.existsSync(config.SSL.cert);
		if (keyExists && certExists) {
			options = {
				key: fs.readFileSync(config.SSL.key),
				cert: fs.readFileSync(config.SSL.cert)
			};
		} else {
			// 操作信息
			var operationInfo = '[createFileServer:]' + '证书文件未找到';
			debugLog.debugOutputLog('server.js:1407', debugLog.debugLevel.DEBUG_LEVEL_INFO, operationInfo);
		}
	} else {
		// 操作信息
		var operationInfo = '[createFileServer:]' + '配置文件未设置证书';
		debugLog.debugOutputLog('server.js:1407', debugLog.debugLevel.DEBUG_LEVEL_INFO, operationInfo);
	}
	return options;
}

/**
 * @method        	createFileServer
 * @description     创建文件服务(创建KMHttp或KMHttps服务)
 * @param           {kmProtocolTypeEnum} 	clsName			协议类名 
 * @param           {String} 				url 			ip + port(如“192.168.2.119:8001”)
 * @param           {Function } 			funcAddress 	http或https处理响应函数
 * @returns         {Boolean}				bRet			创建服务是否成功
 * @example
 * // request为Request对象，与客户端交互；response为Response对象，用于动态响应客户端请示
 * 	function funcAddress(request, response) {
 * 		//...
 * 	}
 *	var serverMng = new Server();
 *	Server.createFileServer(kmProtocolTypeEnum.KM_COMM_HTTP, "192.168.2.119:8001", funcAddress);
 */
Server.prototype.createFileServer = function (clsName, url, funcAddress) {
	var bRet = true;
	var fileServer = null;
	// 获取创建HTTP/HTTPS
	var existClsName = this.typeMng[clsName];
	if (existClsName === undefined) {
		this.typeMng[clsName] = clsName;
		existClsName = clsName;
	}
	// 根据通信协议创建相应的通信通道
	switch (existClsName) {
		case kmProtocolTypeEnum.KM_COMM_HTTP: {
			fileServer = new KMHttp(url, funcAddress);
		}
			break;
		case kmProtocolTypeEnum.KM_COMM_HTTPS: {
			fileServer = new KMHttps(url, funcAddress, options);
		}
			break;
		default:
			break;
	}
	if (fileServer === null) {
		bRet = false;
	}
	return bRet;
};


/**
 * @method        	createDataServer
 * @description     创建数据服务(创建KMWs或KMWss服务)
 * @param           {kmProtocolTypeEnum} 	clsName			协议类名
 * @param           {kmCSTypeEnum} 			csType 			将要创建的是客户端还是服务端
 * @param           {String} 				url 			ip + port(如“192.168.2.119:8000”)
 * @param           {Function } 			funcAddress 	数据服务监听客户端的处理函数
 * @returns         {Boolean}				bRet			创建服务是否成功
 * @example
 * 	function funcAddress(ws) {
 * 		//...
 * 	}
 *	var serverMng = new Server();
 *	Server.createDataServer(kmProtocolTypeEnum.KM_COMM_WS, kmCSTypeEnum.KM_COMM_SERVER, "192.168.2.119:8000", funcAddress);
 */
Server.prototype.createDataServer = function (clsName, csType, url, funcAddress) {
	var bRet = true;
	this.dataServer = null;
	// 获取创建WS/WSS
	var existClsName = this.typeMng[clsName];
	if (existClsName === undefined) {
		this.typeMng[clsName] = clsName;
		existClsName = clsName;
	}
	// 根据通信协议创建相应的通信通道
	switch (existClsName) {
		case kmProtocolTypeEnum.KM_COMM_WS: {
			this.dataServer = new KMWs(csType, url, funcAddress);
		}
			break;
		case kmProtocolTypeEnum.KM_COMM_WSS: {
			this.dataServer = new KMWss(csType, url, funcAddress, options);
		}
			break;
		default:
			break;
	}
	if (this.dataServer === null) {
		bRet = false;
	}
	return bRet;
};

Server.prototype.close = function () {
	this.fileServer.close();
	this.dataServer.close();
};

Server.prototype.getDataServer = function () {
	return this.dataServer;
};


/**
 * @method        	registerSubscribFunc
 * @description     注册回调函数至C++扩展模块
 * @param           {Boolean } 	enable 是否注册
 * @param           {Function } subscribCallback 回调处理函数
 * @returns         {undefined}
 * @example
 * 	function subscribCallback(msg,...value) {
 * 		//...
 * 	}
 *	var serverMng = new Server();
 *	Server.registerSubscribFunc(true, subscribCallback);
 */
Server.prototype.registerSubscribFunc = function (enable, subscribCallback) {
	// 注册回调函数至C++扩展模块
	if (enable) {
		urlRouterObj.registerSubscribFunc(enable, subscribCallback);
	}
};

/**
 * @method        	deleteMarkString
 * @description     删除url中的标识字段
 * @param           {String} 	requestUrl 待处理的url
 * @returns         {String}    resUrl     处理后的url
 */
Server.prototype.deleteMarkString = function (requestUrl) {
	var resUrl = '';
	var requestUrlArr = requestUrl.split('/');
	for (var i = 0; i < requestUrlArr.length; i++) {
		if (requestUrlArr[i].indexOf('FusionWebIDE') !== -1) {
			requestUrlArr[i] = requestUrlArr[i].replace('FusionWebIDE', '');
		}
		if (i < requestUrlArr.length -1) {
			resUrl += requestUrlArr[i] + '/';
		} else {
			resUrl += requestUrlArr[i];
		}
	}
    return resUrl;
}

// http或https处理响应函数
function fileServerHandleFunc(request, response) {
	// 解密，针对url中有中文字符时有必要
	request.url = decodeURI(request.url);
	var requestUrl = undefined;
	// 对url中的标识字段进行过滤
	if (request.url.indexOf('/FusionWebIDE') !== -1) {
		requestUrl = request.url.replace('/FusionWebIDE', '');
		requestUrl = serverMng.deleteMarkString(requestUrl);
	} else {
		requestUrl = request.url;
	}
	// 去除版本号标记字段
	if (requestUrl.indexOf("?v=") !== -1) {
		requestUrl = requestUrl.substr(0, requestUrl.indexOf("?v="));
	}
	// 兼容ie11浏览器  取消其ajax请求数据时默认缓存机制
	if(requestUrl.indexOf("?_=") !== -1){
		requestUrl = requestUrl.substr(0, requestUrl.indexOf("?_="));
	}
	var the_query = url.parse(requestUrl, true).query;
	// 获取文件路径,通过对url的解析
	var pathname = url.parse(requestUrl).pathname;
	// 接收来自于客户端发送的json数据，并做相应处理
	if (request.method === 'GET') {
		// 目前该url的来源未知（文件存在重复加载），有待查证2018.06.07
		if (requestUrl === "/libs/ui/kmui_zh_resource.js") {
			requestUrl = "/DevelopClient/ProjectDevelp" + "/libs/ui/kmui_zh_resource.js";
		}

		// 根据具体的路径（页面名称）加载web页面
		var htmlPath = undefined;
		switch (requestUrl) {
			// 打开登录页面
			case "/": {
				htmlPath = '../../DevelopClient/ProjectDevelp/index.html';
			}
				break;
			case "/help.html": {
				htmlPath = '../../DevelopClient/ProjectDevelp/html/help.html';
			}
				break;
			case "/html/develop.html": {
				htmlPath = '../../DevelopClient/ProjectDevelp/html/develop.html';
			}
				break;
			default:
				break;
		}
		if (htmlPath !== undefined) {
			fileOperateObj.exists(htmlPath, function (exists) {
				if (exists) {
					// 异步读取文件
					fileOperateObj.asynReadFile(htmlPath, 'utf-8', function (err, data) {
						if (err) throw err;
						response.writeHead(200, {
							"Content-Type": "text/html"
						});
						response.write(data);
						response.end();
					});
				} else {
					console.log(htmlPath + 'does not exist!');
					response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
					response.write("the request " + htmlPath + " is not found");
					response.end();
				}
			});
		}

		// 根据具体的路径（用户基本请求）完成相应的信息路由
		var myDate = new Date();
		switch (pathname) {
			case '/getpublickeystring': {
				response.writeHead(200, {
					"Content-Type" : "text/html"
				});
				
				response.end(ServerPublicKeyString);// 将公钥字符串发给客户端
			}
				break;
				// added by yu.sun for 权限配置
			case '/getModuelKey': {
				if (modulekey === null) {
					modulekey = api.generateRSAKey('auz' + currentProName, Bits);
					// 提取出公钥字符串
					moduleString = api.publicKeyString(modulekey);
				}
				response.writeHead(200, {
					"Content-Type": "text/html"
				});

				response.end(moduleString);// 将公钥字符串发给客户端
			}
				break;
                             // end
			case "/login": {
				// 将json字符串转为json对象
				var requestType = the_query["requestType"];
				var timeOut = the_query["timeOut"];
				if (requestType !== undefined) {
					console.log("resquestType: " + requestType);
				} else {
					the_query["requestType"] = 'Login';
					requestType = the_query["requestType"];
				}
				if (timeOut === undefined) {
					the_query["timeOut"] = '3000';
				}
				console.log("data: ");
				console.log(the_query);
				console.log("myDate: " + myDate.toLocaleTimeString());
				var outpwd = api.decrypt(the_query.password, ServerRSAkey);// 解密客户端发来的字符串
				if (outpwd.status === 'success') {
					the_query["password"] = outpwd.plaintext;
				}
				var queryData = JSON.stringify(the_query);
				urlRouterObj.handleFunctionHttp(requestType, response, queryData);
			}
				break;
			case "/logoff": {
				// 将json字符串转为json对象
				var requestType = the_query["requestType"];
				console.log("resquestType: " + requestType);
				console.log("data: ");
				console.log(the_query);
				console.log("myDate: " + myDate.toLocaleTimeString());
				var queryData = JSON.stringify(the_query);
				urlRouterObj.handleFunctionHttp(requestType, response, queryData);
			}
				break;
			case "/changepassword": {
				// 将json字符串转为json对象
				var requestType = the_query["requestType"];
				// 解密客户端发来的旧密码
				var passwordold = the_query["passwordold"];
				// 解密客户端发来的新密码
				var passwordnew = the_query["passwordnew"];
				console.log("resquestType: " + requestType);
				console.log("data: ");
				console.log(the_query);
				console.log("myDate: " + myDate.toLocaleTimeString());
				urlRouterObj.handleFunctionHttp(requestType, response, the_query);
			}
				break;
			case "/getDemoModeUserNum": {
				var obj = {};
				obj["demoModeUserNum"] = demomng.getDemoUserNum();
				response.writeHead(200, "Ok");
				response.write(JSON.stringify(obj), "utf-8");
				response.end();
			}
				break;
			case "/userCheck": {
				var userType = the_query["userType"];
				if(userType === 'demomode'){
					var found = demomng.isVaild(the_query["userID"]);
					var obj = {};
					if(found){
						obj["errorCode"] = 0;
					}
					else{
						obj["errorCode"] = 1;
					}
					response.writeHead(200, "Ok");
					response.write(JSON.stringify(obj), "utf-8");
					response.end();
				}
				else{
					var requestType = the_query["requestType"];
					var queryData = JSON.stringify(the_query);
					urlRouterObj.handleFunctionHttp(requestType, response, queryData);
				}
			}
				break;
			//add Web GIs
			case "/TILE": {
				var queryObj = the_query;
				if (queryObj.type) {
					var fileFolder = config.tilePath;
					
					if (!fs.existsSync(fileFolder)){
						response.writeHead(200, { 'content-type': 'text' });
						response.write(JSON.stringify({type:"error"}));
						response.end();
						return;
					}

					var tileFile = new TileFileAsync(fileFolder);
					switch (queryObj.type) {
						case "getArea": {
							tileFile.getTileAreas(tileAreas => {
								response.writeHead(200, { 'content-type': 'text' });
								response.write(JSON.stringify(tileAreas));
								response.end();
							});
						}
							break;

						case "initial": {
							let tilePath = queryObj.tilePath;
							tileFile = new TileFileAsync(pathModule.join(fileFolder, tilePath));
							var level = parseInt(queryObj.level);
							tileFile.getTileRangeAsync(level, tileRange => {
								response.writeHead(200, { 'content-type': 'text' });
								response.write(JSON.stringify(tileRange));
								response.end();
							});
						}
							break;

						case "queryLevelRange": {
							let tilePath = queryObj.tilePath;
							tileFile = new TileFileAsync(pathModule.join(fileFolder, tilePath));
							var level = queryObj.level;
							tileFile.getLevelTileRangeAsync(level, (tileRange) => {
								response.writeHead(200, { 'content-type': 'text' });
								response.write(JSON.stringify(tileRange));
								response.end();
							});
						}
							break;
						case "queryInfo": {
							tileFile.getTileDataAsync(queryObj.queryCond, tileDatas => {
								response.writeHead(200, { 'content-type': 'text' });
								response.write(JSON.stringify(tileDatas));
								response.end();
							});
						}
							break;
						default: {

						}
							break;
					}

				}

			}
				break;
			// add by qiankun.xie  2019.06.25
			case "/PageGroups": {
				// 获取页面组加密状态信息
				var pageGroupEncryptInfo = fileEncryptObj.getEncryptInfo();
				// 向客户端返回加密状态数据
				response.writeHead(200, "Ok");
				response.write(JSON.stringify(pageGroupEncryptInfo), "utf-8");
				response.end();
			}
				break;
			// end by qiankun.xie
			default:
				break;
		}

		// 根据具体的路径获取图表信息
		if (pathname.indexOf("porfesstionalEcharts") !== -1) {
			// 文件信息列表
			var filesList = [];
			// 一级文件夹信息列表
			var oneDirectoryList = [];
			// 二级文件夹信息列表
			var twoDirectoryList = [];
			// 获取图表文件信息枚举
			fileOperateObj.chartFileInfoDisplay("./Predefined/porfesstionalEcharts", filesList, oneDirectoryList, twoDirectoryList);
			// console.log(filesList);
			// console.log(oneDirectoryList);
			// console.log(twoDirectoryList);
			// 根据具体的图表信息获取类型进行相应的操作
			var groupObj = {};
			var nongroupArr = [];
			var textStr = "";
			var JsonString = undefined;
			var pngCount = 0;
			switch (pathname) {
				// 不分组
				case "/porfesstionalEcharts:nongroup": {
					for (var i = 0; i < twoDirectoryList.length; i++) {
						var objTwo = twoDirectoryList[i];
						for (var j = 0; j < filesList.length; j++) {
							if (filesList[j].path === objTwo.path) {
								if (filesList[j].filename.indexOf("png") !== -1) {
									var obj = {};
									obj.chartName = objTwo.filename;
									obj.url = filesList[j].path + '/' + filesList[j].filename;
									nongroupArr.push(obj);
								} else {
									pngCount++;
								}
							} else {
								pngCount++;
							}
							if (pngCount === filesList.length) {
								var obj = {};
								obj.chartName = objTwo.filename;
								obj.url = "res/nopicture.png";
								nongroupArr.push(obj);
							}
						}
					}
					// console.log(nongroupArr);
					JsonString = JSON.stringify(nongroupArr);
					response.writeHead(200, {
						"Content-Type": "application/json"
					});
					response.writeHead(200, "Ok");
					response.write(JsonString, "utf-8");
					response.end();
				}
					break;
				// 分组
				case "/porfesstionalEcharts:group": {
					for (var i = 0; i < oneDirectoryList.length; i++) {
						var objOne = oneDirectoryList[i];
						var arr = [];
						for (var j = 0; j < twoDirectoryList.length; j++) {
							var objTwo = twoDirectoryList[j];
							if (objTwo.path.indexOf(objOne.path + '/') !== -1) {
								for (var m = 0; m < filesList.length; m++) {
									if (filesList[m].path === objTwo.path) {
										var obj = {};
										if (filesList[m].filename.indexOf("png") !== -1) {
											obj.chartName = objTwo.filename;
											var picPath = filesList[m].path + '/' + filesList[m].filename;
											//读取图片Base64格式数据
											var imgData = fileOperateObj.syncReadFile(picPath);
											var imgBase64 = imgData.toString('base64');
											obj.img = "data:image/png;base64," + imgBase64;
											for (var n = 0; n < filesList.length; n++) {
												if (filesList[n].path === filesList[m].path) {
													if (filesList[n].filename.indexOf("png") === -1) {
														var optionPath = filesList[n].path + '/' + filesList[n].filename;
														obj.option = fileOperateObj.syncReadFile(optionPath, 'utf-8');
													}
												}
											}
											arr.push(obj);
										}
										else {
											pngCount++;
										}
									} else {
										pngCount++;
									}
									// if (pngCount === filesList.length) {
									// 	var obj = {};
									// 	obj.chartName = objTwo.filename;
									// 	obj.url = "res/nopicture.png";
									// 	arr.push(obj);
									// }
								}
							}
						}
						groupObj[objOne.filename] = arr;
					}
					// console.log(groupObj);
					var JsonString = JSON.stringify(groupObj);
					response.writeHead(200, {
						"Content-Type": "application/json"
					});
					response.writeHead(200, "Ok");
					response.write(JsonString, "utf-8");
					response.end();
				}
					break;
				// 文本
				default: {
					var realPath = "." + pathname;
					fileOperateObj.exists(realPath, function (exists) {
						if (exists) {
							// 获取文件状态，根据状态进行相应的文件操作
							fileOperateObj.getFileInfo(realPath, function (err, stats) {
								if (err) {
									response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
									response.write("the request " + realPath + " is not found");
									response.end();
								} else {
									response.writeHead(200, {
										"Content-Type": "application/json"
									});
									// 同步读取文件
									textStr = fileOperateObj.syncReadFile(realPath, "utf-8");
									response.writeHead(200, "Ok");
									response.write(textStr, "utf-8");
									response.end();
								}
							});
						} else {
							console.log(path + ' does not exist!');
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + path + " is not found");
							response.end();
						}
					});	
				}
					break;
			}
		}

		//add Web Gis
		if (pathname.indexOf("tile") !== -1) {
			var tilePath = pathname.toLocaleLowerCase();
			if (tilePath.startsWith('/tile')) {
				var relativePath = pathModule.join(config.tilePath, tilePath.substr(5, tilePath.length));
				fileOperateObj.exists(relativePath, function (exists) {
					if (exists) {
						fs.readFile(relativePath, (err, data) => {
							if (err) {
								response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
								response.end("file not found!");

							}
							response.writeHead(200, { 'content-type': mime.lookup(relativePath) });
							response.write(data);
							response.end();
						});
					}
					else {
						console.log(relativePath + ' does not exist!');
						response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
						response.write("the request " + relativePath + " is not found");
						response.end();
					}
				});
			};
		}
		// add for web gis end.

		// 获取文件路径中的后缀名
		var ext = pathname.match(/(\.[^.]+|)$/)[0];
		// 根据具体的后缀名来读取相应的文件(css/js/gif/png/ico)
		// console.log(requestUrl);
		switch (ext) {
			case ".map":
				{
					var path = "../../" + requestUrl;
					fileOperateObj.exists(path, function (exists) {
						if (exists) {
							// 异步读取文件
							fileOperateObj.asynReadFile(path, 'utf-8', function (err, data) {
								if (err) throw err;
								response.writeHead(200, {
									"Content-Type": "application/json"
								});
								response.write(data);
								response.end();
							});
						} else {
							console.log(path + ' does not exist!');
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + path + " is not found");
							response.end();
						}
					});
				}
				break;
			case ".css":
			case ".js":
				{
					var path = "../../" + requestUrl;
					fileOperateObj.exists(path, function (exists) {
						if (exists) {
							// 异步读取文件
							fileOperateObj.asynReadFile(path, 'utf-8', function (err, data) {
								if (err) throw err;
								response.writeHead(200, {
									"Content-Type": {
										".css": "text/css",
										".js": "application/javascript",
									}[ext]
								});
								response.write(data);
								response.end();
							});
						} else {
							console.log(path + ' does not exist!');
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + path + " is not found");
							response.end();
						}
					});
				}
				break;
			case ".json":
				{
					var path = "../.." + requestUrl;
					fileOperateObj.exists(path, function (exists) {
						if (exists) {
							// 获取文件状态，根据状态进行相应的文件操作
							fileOperateObj.getFileInfo(path, function (err, stats) {
								if (err) {
									response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
									response.write("the request " + path + " is not found");
									response.end();
								} else {
									response.writeHead(200, {
										"Content-Type": "application/json"
									});
									// 同步读取文件
									var content = fileOperateObj.syncReadFile(path, "utf-8");
									if (path.indexOf("GraphicModelManager.json") !== -1) {
										var contentObj = JSON.parse(content);
										var pathDefaultGraphicModel = '../../DevelopServer/Server/Config/km_default_GraphicModel.json'
										fileOperateObj.getFileInfo(pathDefaultGraphicModel, function (err, stats) {
											if (err) {
												response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
												response.write("the request " + pathDefaultGraphicModel + " is not found");
												response.end();
											} else {
												// 获取默认基本图形模型库中的内容（json串格式）
												var defaultGraphicModelJson = fileOperateObj.syncReadFile(pathDefaultGraphicModel, "utf-8");
												// 获取json对象格式
												var defaultGraphicModelJsonObj = JSON.parse(defaultGraphicModelJson);
												// 获取具体的默认内容
												var defaultStr = JSON.stringify(defaultGraphicModelJsonObj.基本图形模型库.UI控件.mode) +
													JSON.stringify(defaultGraphicModelJsonObj.基本图形模型库.布局容器.mode);
												// 获取当前配置文件中关于默认内容相关的内容
												var currentStr = JSON.stringify(contentObj.基本图形模型库.UI控件.mode) +
													JSON.stringify(contentObj.基本图形模型库.布局容器.mode);
												if (defaultStr === currentStr) {
													response.writeHead(200, {
														"Content-Type": "application/json"
													});
													response.writeHead(200, "Ok");
													response.write(content, "utf-8");
													response.end();
												} else {
													contentObj.基本图形模型库.UI控件.mode = defaultGraphicModelJsonObj.基本图形模型库.UI控件.mode;
													contentObj.基本图形模型库.布局容器.mode = defaultGraphicModelJsonObj.基本图形模型库.布局容器.mode;
													contentObj = JSON.stringify(contentObj);
													var result = fileOperateObj.syncWriteFile(1, path, contentObj, 'utf-8');
													response.writeHead(200, {
														"Content-Type": "application/json"
													});
													response.writeHead(200, "Ok");
													response.write(contentObj, "utf-8");
													response.end();
												}
											}
										})
									// added by yu.sun 模块权限 2019-07-15
									} else if (path.indexOf("PageConfig.json") !== -1 || path.indexOf("pageConfig.json") !== -1 ) {
										// 判断当前pageConfig是否处于加密状态
										if (content.indexOf("pageGroupMng") === -1) {
											// 加密状态
											try {
												content = KMEncryption.decrypt(content, "pageConfig");
											} catch(err) {
												content = content.substring(1, content.length - 1);
												content = KMEncryption.decrypt(content, "pageConfig");
											}
										}
										response.writeHead(200, "Ok");
										response.write(content, "utf-8");
										response.end();
									}
									// end 
									// add by yaping.feng for 漏洞修复 at 2021.11.1
									else if(path.indexOf("ProjectDes.json") !== -1) {
										var contentObj = JSON.parse(content);
										var resObj = {authority: contentObj.authority};
										content = JSON.stringify(resObj);
										response.writeHead(200, "Ok");
										response.write(content, "utf-8");
										response.end();
									}
									// end by yaping.feng
									else {
										response.writeHead(200, "Ok");
										response.write(content, "utf-8");
										response.end();
									}
								}
							});
						} else {
							console.log(path + ' does not exist!');
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + path + " is not found");
							response.end();
						}
					});
				}
				break;
			case ".delete":
				{
					var strArray = requestUrl.split('.');
					var path = "../.." + strArray[0] + ".json";
					fileOperateObj.exists(path, function (exists) {
						if (exists) {
							// 获取文件状态，根据状态进行相应的文件操作
							fileOperateObj.getFileInfo(path, function (err, stats) {
								if (err) {
									response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
									response.write("the request " + path + " is not found");
									response.end();
								} else {
									// 删除相应文件
									fileOperateObj.deleteFile(path, function (err) {
										if (err) {
											console.log(err);
											response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
											response.write("the request " + path + " is delete failed");
											response.end();
										} else {
											response.writeHead(200, "OK");
											response.write("the request " + path + " is delete success");
											response.end();
										}
									});
								}
							});
						} else {
							console.log(path + ' does not exist!');
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + path + " is not found");
							response.end();
						}
					});
				}
				break;
			case ".batchdelete":
				{
					var strArray = requestUrl.split('.');
					var desArray = strArray[0].split('/');
					var filesArrayStr = desArray[desArray.length - 1];
					var index = strArray[0].indexOf(filesArrayStr);
					var filePath = strArray[0].substr(0, index);
					var filesArray = JSON.parse(filesArrayStr);
					for (var i = 0; i < filesArray.length; ++i) {
						(function (i) {
							var path = "../.." + filePath + filesArray[i] + ".json";
							fileOperateObj.exists(path, function (exists) {
								if (exists) {
									// 获取文件状态，根据状态进行相应的文件操作
									fileOperateObj.getFileInfo(path, function (err, stats) {
										if (err) {
											// response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
											// response.write("the request " + path + " is not found");
											// response.end();
										} else {
											// 删除相应文件
											fileOperateObj.deleteFile(path, function (err) {
												if (err) {
													console.log(err);
													// response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
													// response.write("the request " + path + " is delete failed");
													// response.end();
												} else {
													// response.writeHead(200, "OK");
													// response.write("the request " + path + " is delete success");
													// response.end();
												}
											});
										}
									});

								} else {
									console.log(path + ' does not exist!');
									// response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
									// response.write("the request " + path + " is not found");
									// response.end();
								}
							});
						})(i);
					}
					response.writeHead(200, "OK");
					response.write("the request " + filesArrayStr + " is delete success");
					response.end();
				}
				break;
			case ".bmp":
			case ".gif":
			case ".jpg":
			case ".png":
			case ".svg":
				{
					//add Web Gis
					if (pathname.indexOf("tile") === -1) {
						if (ext === ".gif") {
							response.writeHead(200, {
								"Content-Type": "image/gif"
							});
						}
						else if (ext === ".jpg") {
							response.writeHead(200, {
								"Content-Type": "image/jpeg"
							});
						}
						else if (ext === ".png") {
							response.writeHead(200, {
								"Content-Type": "image/png"
							});
						}
						else if (ext === ".bmp") {
							response.writeHead(200, {
								"Content-Type": "image/bmp"
							});
						}
						else if (ext === ".svg") {
							response.writeHead(200, {
								"Content-Type": "image/svg+xml"
							});
						}
						//兼容工程名为‘Project’ + KF工程库Guid
						if (currentProName.length == 0) {
							var projectConfigPath = '../../DevelopServer/Server/Projects/ProjectManager.json';
							fileOperateObj.exists(projectConfigPath, function (exists) {
								if (exists) {
									// 此段代码mini时不识别，故修改
									var content = fileOperateObj.syncReadFile(projectConfigPath, "utf-8");
									if (typeof content === "string") {
										content = JSON.parse(content)
									}
									currentProName = 'Project-' + content["default"];
									readProPicture();
								} else {
									console.log(projectConfigPath + ' :does not exist!');
								}
							});
						} else {
							readProPicture();
						}

					function readProPicture() {
						var ProjectPath = "/Projects/" + currentProName + "/res/images/";
						// 去除多于的路径信息// src中图片缺少的路径信息
						var lessPath = "/DevelopClient/ProjectDevelp";
							// 使用easyui图片文件
							var easyuiPicPath = "iconimages";
						// 第三方库对应的图片路径特征
						var ThirdPartyPath = "ThirdParty";
						// if (requestUrl.indexOf(lessPath) === -1 && requestUrl.indexOf(ThirdPartyPath) === -1) {
						// 	requestUrl = lessPath + requestUrl;
						// }
						var path = " ";
							var picName = unescape(requestUrl);
							//针对工程路径下的图片
							if (picName.indexOf(ProjectPath) !== -1) {
								path = "../.." + picName;
							} else if (picName.indexOf(ThirdPartyPath) !== -1) {
								path = "../.." + picName;
							} else if (picName.indexOf(lessPath) !== -1) {
								path = "../.." + picName;
							} else if (picName.indexOf(easyuiPicPath) !== -1) {
                                path = "../.." + picName;
							} else {
								path = "../.." + lessPath + picName;
							}
						// var path = "../.." + requestUrl;
						fileOperateObj.exists(path, function (exists) {
							if (exists) {
								// 同步读取文件
								var content = fileOperateObj.syncReadFile(path, "binary"); //格式必须为 binary 否则会出错  
									// response.writeHead(200, "Ok");
								response.write(content, "binary");
								response.end();
							} else {
								console.log(path + ' does not exist!');
								response.writeHead(404, "not found", {
									'Content-Type': 'text/plain'
								});
								response.write("the request " + path + " is not found");
								response.end();
							}
						});
					}
				}
				}
				break;
			case ".ico":
				{
					if (requestUrl.indexOf("favicon") === -1) {
						response.writeHead(200, {
							"Content-Type": "image/ico"
						});

						// 去除多于的路径信息// src中图片缺少的路径信息
						var lessPath = "/DevelopClient/ProjectDevelp";
						// 第三方库对应的图片路径特征
						var ThirdPartyPath = "ThirdParty";
						if (requestUrl.indexOf(lessPath) === -1 && requestUrl.indexOf(ThirdPartyPath) === -1) {
							requestUrl = lessPath + requestUrl;
						}

						var path = "../.." + requestUrl;
						fileOperateObj.exists(path, function (exists) {
							if (exists) {
								// 同步读取文件
								var content = fileOperateObj.syncReadFile(path, "binary");	//格式必须为 binary 否则会出错  
								response.writeHead(200, "Ok");
								response.write(content, "binary");
								response.end();
							} else {
								console.log(path + ' does not exist!');
								response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
								response.write("the request " + path + " is not found");
								response.end();
							}
						});
					}
				}
			default:
				{
				}
				break;
		}
	} else if (request.method === 'POST') {
		var chunks = [];
		var size = 0;
		//兼容输入流中有宽字节编码的乱码的问题
		request.on('data', function (chunk) {
			chunks.push(chunk);
			size += chunk.length;
		});
		request.on('end', function () {
			var buf = Buffer.concat(chunks, size);
			var postdata = iconv.decode(buf, 'utf8');
			// console.log(postdata);
			var pathRequest = requestUrl;
			if (requestUrl === "/DevelopServer/Server/Projects/") {
				var path = "../.." + requestUrl;
				if (postdata.indexOf('FusionWebIDE/') !== -1) {
					postdata = postdata.replace('FusionWebIDE/', '');
				}
				// 复制目录
				if (postdata.indexOf("+") > -1) {
					var arr = postdata.split('+');
					var oldProGuid = arr[0];
					var newProGuid = arr[1];
					currentProName = "Project-" + newProGuid;
					var newProPath = path + currentProName;
					// global.deployExsit = true;
					if (global.deployExsit !== undefined) {
						if (global.deployExsit === false) {
							console.log('当前有人在部署');
							response.writeHead(200, "OK");
							response.write("正在部署");
							response.end();
						}
					}
					global.deployExsit = false;
					global.deployingExist = arr[2];
					var arrChange = [];
					fileOperateObj.exists(newProPath, function (exists) {
						if (exists) {
							// fileOperateObj.FileDelete(newProPath);
							// fileOperateObj.FileExist(path + "DefaultProject", newProPath, fileOperateObj.FileCopy);
						} else {
							// var arrNew = newProPath.replace(currentProName,currentProName + 'ProjectsdeployNew')
							// fileOperateObj.CreateFolder(newProPath);
							// fileOperateObj.CreateFolder(arrNew);
							global.srcPath = newProPath;
							//fileOperateObj.CreateFolder(newProPath);
							global.srcPathCount = fileOperateObj.findCountSync(path + "DefaultProject");
							fileOperateObj.FileExist(path + "DefaultProject",newProPath,arrChange,function(err){
							// fileOperateObj.FileExist(path + "DefaultProject", arrNew, fileOperateObj.FileCopy, countOld, arrNew, function (err) {
								if (err) {
									console.log(err);
									global.deployExsit = true;
									global.deployingExist = undefined;
									response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
									response.write("the request is copy failed");
									response.end();
								} else {
									global.deployExsit = true;
									global.deployingExist = undefined;
									response.writeHead(200, "OK");
									response.write("部署成功");
									response.end();
								}
							});
						}
					});
				} else if (postdata.indexOf("-") > -1) {
					var arr = postdata.split('-');
					// 首先创建一个json文件，作为工程部署的标志
					// fileOperateObj.FileDelete(arr[1]);
					// var arrNew = arr[1].replace('Projects','ProjectsdeployNew')
					if (global.deployExsit !== undefined) {
						if (global.deployExsit === false) {
							console.log('当前有人在部署');
							response.writeHead(200, "OK");
							response.write("正在部署");
							response.end();
							return;
						}
					}
					global.deployExsit = false;
					global.deployingExist = arr[2];
					// 首先获取所有文件的修改时间，并记录
					var arrChange = [];
					fileOperateObj.FileCopyJson(arr[0] + 'ProjectManager.json', arr[1] + 'ProjectManager.json', function (err) {
						if (err) {
							//throw err
						}
						var projectDeployTxt = fs.readFileSync(arr[0] + 'ProjectManager.json', 'utf-8');
						var projectDeployPath = eval("(" + projectDeployTxt + ")");
						var countOld = fileOperateObj.findCountSync(arr[0] + 'Project-' + projectDeployPath.default);
						var oldPath = arr[0] + 'Project-' + projectDeployPath.default;
						var newPath = arr[1] + 'Project-' + projectDeployPath.default;
						global.srcPath = newPath;
						global.srcPathCount = fileOperateObj.findCountSync(arr[0] + 'Project-' + projectDeployPath.default);
						console.log('执行是否正确'+global.srcPathCount)
						fileOperateObj.FileExist(oldPath,newPath,arrChange,function(err){
							if (err) {
								global.deployExsit = true;
								global.deployingExist = undefined;
								console.log(err);
								response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
								response.write("the request is copy failed");
								response.end();
							} else {
								// add by xiaojing.han for【thirdlogin通用化处理】thirdlogin.js文件部署到运行态
								fileOperateObj.exists(arr[0] + 'thirdlogin.js', function(exists){
									if(exists){
										fileOperateObj.FileCopyJson(arr[0] + 'thirdlogin.js',arr[1] + 'thirdlogin.js',function(err){
											if (err) {
												global.deployExsit = true;
												global.deployingExist = undefined;
												console.log(err);
												response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
												response.write("the request is copy failed");
												response.end();
											} else {
												global.deployExsit = true;
												global.deployingExist = undefined;
												response.writeHead(200, "OK");
												response.write("部署成功");
												response.end();
											}
										});
									} else {
										console.log("thirdlogin.js is not found");
										global.deployExsit = true;
										global.deployingExist = undefined;
										response.writeHead(200, "OK");
										response.write("部署成功");
										response.end();
									}

								});
							}
						});
					})
				}
				// added by yu.sun 2019-07-11 for 校验密码正确性
			} else if (pathname.indexOf("comparePassWord") !== -1) {
				response.writeHead(200, "OK");
				postdata = decodeURIComponent(postdata);
				postdata = JSON.parse(postdata)
				var newpassword = api.decrypt(postdata.password[0].newPW, modulekey);
				var oldpassword = api.decrypt(postdata.password[0].oldPW, modulekey);
				if (newpassword.plaintext === oldpassword.plaintext) {
					response.write("true");
				} else {
					response.write("false");
				}
				response.end();
				// 将公钥字符串发给客户端
			}
			// end
			 else if (requestUrl === "/setDemoMode") {
				var postdataObj = JSON.parse(postdata);
				if (postdataObj["type"] === 'add') {
					demomng.add(postdataObj["guid"]);
				} else if (postdataObj["type"] === 'remove') {
					demomng.remove(postdataObj["guid"]);
				}
				response.writeHead(200, {
					"Content-Type": "application/json"
				});
				response.write('{ "name": "success"}', "utf-8");
				response.end();
			} else if (requestUrl === "/setModelCopyInfo") {
				// 对象请求类型枚举
				var kmTypeEnum = {
					GRAPHICMODEL_BASE: 0,	// 基本图形模型   
					PROFESSIONALMODEL_BASE: 1,    // 专业模型
					UI: 2,	// 布局控件  
					LAYOUT: 3,	// UI控件                   
					GRAPHICMODEL_ASSOCIATED: 4,	// 已关联图形模型  
					GRAPHICMODEL_COMBINED: 5,	// 组合图形模型  
					DEVELOP_PAGE: 6,	// 页面
					GRAPHICGROUP: 7,	// 组合图素
				};
				var data = JSON.parse(postdata);
				var path = ""
				switch (data.modelType) {
					case kmTypeEnum.GRAPHICMODEL_COMBINED: { // 组合模型
						path = "./Projects/" + currentProName + "/res/graphicModel_combined"
					}
						break;
					case kmTypeEnum.GRAPHICMODEL_ASSOCIATED: {
						path = "./Projects/" + currentProName + "/res/graphicModel_associated" //关联模型
					}
						break;
					case kmTypeEnum.GRAPHICMODEL_BASE: {
						path = "./Projects/" + currentProName + "/res/graphicModel_base" //基本图形模型
					}
						break;
					case kmTypeEnum.PROFESSIONALMODEL_BASE: {
						path = "./Projects/" + currentProName + "/res/professionalModel_base"; //专业模型
					}
						break;
					case kmTypeEnum.GRAPHICGROUP: {
						path = "./Predefined/graphicGroup"; //组合图素
					}
						break;
					default:
						break;
				}
				if (path == "") {
					response.writeHead(200, {
						"Content-Type": "application/json"
					});
					response.write('{ "type": "wrong"}', "utf-8");
					response.end();
				} else {
					for (var i = 0; i < data.modelNameArr.length; i++) {
						var readPath = path + "/" + data.modelNameArr[i] + ".json";
						var writePath = path + "/" + data.copyNameArr[i] + ".json";
						var dataWrite = fileOperateObj.syncReadFile(readPath, 'utf-8');
						if(data.modelType !== kmTypeEnum.GRAPHICGROUP) {
							dataWrite = JSON.parse(dataWrite);
							dataWrite.modlePrototype.name = data.copyNameArr[i];
							dataWrite.pageProperty.name = data.copyNameArr[i];
							dataWrite = JSON.stringify(dataWrite)
						}
						if (dataWrite !== undefined) {
							fileOperateObj.syncWriteFile(1, writePath, dataWrite, 'utf-8');
						}
					}
					response.writeHead(200, {
						"Content-Type": "application/json"
					});
					response.write('{ "type": "success"}', "utf-8");
					response.end();
				}
			}
			// add by qiankun.xie  2019.06.27
			else if (requestUrl === "/Encrypt/modulencryption") {
				// 创建数据库对象
				if(config.redisConnection){
					redisDBObj.createClient(config.redisConnection.redis_ip, config.redisConnection.redis_port);
				}else{
					redisDBObj.createClient("127.0.0.1", 6379);
				}

				// 将收到的json串转化成json对象
				var jsonPostdata = JSON.parse(postdata);

				// 从redis数据库获取工程密码：若存在，返回密码；不存在，返回null
				redisDBObj.getValue(jsonPostdata.projectID, getProjectIDCallBack);
				function getProjectIDCallBack(err, existedPassWord) {
					if (err) {
						response.writeHead(404, "not found");
						response.write("read database error");
						response.end();
						return;
					}
					if (existedPassWord === null) {
						// 设置工程密码
						redisDBObj.setValue(jsonPostdata.projectID, jsonPostdata.encryptedPassword);
					} else {
						if (existedPassWord !== jsonPostdata.encryptedPassword) {
							// 密码错误，直接返回
							// 向客户端返回加密状态数据
							response.writeHead(200, "Ok");
							response.write("password error", "utf-8");
							response.end();
							return;
						}
					}

					// 将指令中的false变为true，表示要为这些页面加密
					for (var i = 0; i < jsonPostdata.encryStateArr.length; i++) {
						jsonPostdata.encryStateArr[i] = true;
					}

					// 对选择的页面进行加密
					var pageGroupEncryptInfo = fileEncryptObj.fileEncrypt(jsonPostdata);
					// 去除密码
					delete pageGroupEncryptInfo.encryptedPassword;

					// 向客户端返回加密状态数据
					response.writeHead(200, "Ok");
					response.write(JSON.stringify(pageGroupEncryptInfo), "utf-8");
					response.end();
				}
				return;
			} else if (requestUrl === "/Encrypt/moduldecryption") {
				// 创建数据库对象
				if(config.redisConnection){
					redisDBObj.createClient(config.redisConnection.redis_ip, config.redisConnection.redis_port);
				}else{
					redisDBObj.createClient("127.0.0.1", 6379);
				}

				// 将收到的json串转化成json对象
				var jsonPostdata = JSON.parse(postdata);

				// 从redis数据库获取工程密码：若存在，返回密码；不存在，返回null
				redisDBObj.getValue(jsonPostdata.projectID, getProjectIDCallBack);
				function getProjectIDCallBack(err, existedPassWord) {
					if (err) {
						response.writeHead(404, "not found");
						response.write("read database error");
						response.end();
						return;
					}
					if (existedPassWord === null) {
						// 设置工程密码
						redisDBObj.setValue(jsonPostdata.projectID, jsonPostdata.encryptedPassword);
					} else {
						if (existedPassWord !== jsonPostdata.encryptedPassword) {
							// 密码错误，直接返回
							// 向客户端返回解密状态数据
							response.writeHead(200, "Ok");
							response.write("password error", "utf-8");
							response.end();
							return;
						}
					}

					// 将指令中的true变为false，表示要为这些页面解密
					for (var i = 0; i < jsonPostdata.encryStateArr.length; i++) {
						jsonPostdata.encryStateArr[i] = false;
					}

					// 对选择的页面进行解密
					var pageGroupEncryptInfo = fileEncryptObj.fileDecrypt(jsonPostdata);
					// 去除密码
					delete pageGroupEncryptInfo.encryptedPassword;
					
					// 获取所有页面加密状态
					// 标记是否所有页面都已解密
					var isAllDecrypted = true;
					var newGroupEncryptInfo = fileEncryptObj.getEncryptInfo();
					//-var newGroupEncryptInfo = fileEncryptObj.getEncryptInfo(jsonPostdata.projectID);
					for (var i = 0; i < newGroupEncryptInfo.encryStateArr.length; i++) {
						if (newGroupEncryptInfo.encryStateArr[i] === true) {
							isAllDecrypted = false;
							break;
						}
					}

					if (isAllDecrypted) {
						// 删除redis数据库中的工程ProjectID字段
						redisDBObj.delKey(jsonPostdata.projectID);
					}

					// 向客户端返回解密状态数据
					response.writeHead(200, "Ok");
					response.write(JSON.stringify(pageGroupEncryptInfo), "utf-8");
					response.end();
				}
				return;
			}
			// end by qiankun.xie
			// 获取文件路径中的后缀名
			var ext = pathname.match(/(\.[^.]+|)$/)[0];
			// 根据具体的后缀名来读取相应的文件(css/js/gif/png/ico)
			var result = -1;
			switch (ext) {
				case ".json":
					{
						var realPath = "../.." + requestUrl;
						// added by yu.sun 模块权限 2019-07-15
						if (realPath.indexOf("PageConfig.json") !== -1) {
							if(typeof(postdata) === 'object') {
								postdata = JSON.stringify(postdata)
							}
							// var postdata = JSON.parse(postdata)
							// postdata.authorityShow = ''
							// postdata = JSON.stringify(postdata)
							postdata = KMEncryption.encrypt(postdata, "pageConfig");
							fileOperateObj.syncWriteFile(1, realPath, JSON.stringify(postdata), 'utf-8');
						} else if(realPath.indexOf("ProjectAuz.json") !== -1) {
                                                         if(typeof(postdata) === 'string') {
								postdata = JSON.parse(postdata);
							}
							var key = KMEncryption.encrypt(JSON.stringify(postdata.pageGroupAuz), "projectAuz");
							postdata.key = key;
							result = fileOperateObj.syncWriteFile(1, realPath, JSON.stringify(postdata), 'utf-8');
					        }else {
							result = fileOperateObj.syncWriteFile(1, realPath, postdata, 'utf-8');
					        }
						// end

						if (!result) {
							console.log(pathname + ": Save Success");
							response.writeHead(200, {
								"Content-Type": "application/json"
							});
							response.writeHead(200, "Ok");
							response.write('true', "utf-8");
							response.end();
						} else {
							console.log(pathname + ": Save Failed");
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + realPath + " is not found");
							response.end();
						}
					}
					break;
					//added by yu.sun 设置easyui图标css数据
					case ".css":
					{
						var realPath = "../.." + requestUrl;
						result = fileOperateObj.syncWriteFile(1, realPath, postdata, 'utf-8');
						if (!result) {
							console.log(pathname + ": Save Success");
							response.writeHead(200, {
								"Content-Type": "application/json"
							});
							response.writeHead(200, "Ok");
							response.write('{ "name": "success"}', "utf-8");
							response.end();
						} else {
							console.log(pathname + ": Save Failed");
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + realPath + " is not found");
							response.end();
						}
					}
					break;
					// end 
				case ".bmp":
				case ".gif":
				case ".jpg":
				case ".png":
				case ".svg":
					{
						var pictureName = unescape(pathname);
						var kkll = decodeURIComponent(postdata);
						//过滤data:URL
						if (ext === ".svg") {
							var base64Data = kkll.replace(/^data:image\/svg\+xml;base64,/, "");
						} else {
							var base64Data = kkll.replace(/^data:image\/\w+;base64,/, "");
						}
						// base64Data = base64Data.replace(/s/g, "+");
						var dataBuffer = new Buffer(base64Data, 'base64');
						var ProjectPath = "Projects/" + currentProName + "/res/images/";
						//工程路径
						if (pictureName.indexOf(ProjectPath) !== -1) {
							pictureName = "." + pictureName;
						}
						else {
							pictureName = "../.." + pictureName;
						}
						//软件图标资源路径
						result = fileOperateObj.syncWriteFile(1, pictureName, dataBuffer);
						if (!result) {
							console.log(pictureName + ": Save Success");
							response.writeHead(200, {
								"Content-Type": "application/json"
							});
							response.writeHead(200, "Ok");
							response.write('{ "name": "success"}', "utf-8");
							response.end();
						} else {
							console.log(pictureName + ": Save Failed");
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + realPath + " is not found");
							response.end();
						}
					}
					break;
				case ".ico":
					{

					}
					break;
				case ".rename": {
					var strArray = requestUrl.split('.');
					var oldAndNewFileName = strArray[0];
					var fileArray = oldAndNewFileName.split('/');
					var pathArray = fileArray.slice(1, fileArray.length - 2);
					var path = pathArray.join('/');
					var oldFilePath = '../../' + path + '/' + fileArray[fileArray.length - 2] + '.json';
					var newFilePath = '../../' + path + '/' + fileArray[fileArray.length - 1] + '.json';

					result = fileOperateObj.rename(oldFilePath, newFilePath, function (err) {
						if (err) {
							response.writeHead(404, "not found", { 'Content-Type': 'text/plain' });
							response.write("the request " + oldFilePath + " is not found");
							response.end();
						} else {
							response.writeHead(200, {
								"Content-Type": "application/json"
							});
							response.write('{ "name": "success"}', "utf-8");
							response.end();
						}
					});
				}
					break;
				default:
					{
					// 	console.log(pathname + ": Save Success");
					// 	response.writeHead(200, {
					// 		"Content-Type": "application/json"
					// 	});
					// 	response.write('{ "name": "success"}', "utf-8");
					// 	response.end();
					}
					break;
			}
		});
	}
}


// 数据服务监听客户端的处理函数
function dataServerHandleFunc(ws, req) {
	var location = url.parse(req.url, true);
	var clientIp = req.connection.remoteAddress;
	// 客户端数据处理函数
	function dataServerReceiveMsgFunc(syncType, ws, msg) {
		if (msg === 'ping') {
			var operationInfo = '[dataServerReceiveMsgFunc]: ' + "接收到客户端发送的心跳检测信息： " + msg;
			// console.log(operationInfo);
			if (ws.readyState === 1) {
				ws.send('pang');
			}
		} else {
			var resquestType = undefined;
			var jsonMsg = msg;
			var jsonMsg = JSON.parse(msg);
			if (syncType) {	// 同步处理
			} else {// 异步处理
				// 获取请求类型
				resquestType = jsonMsg.requestType;
				// 操作信息
				if (resquestType === "InitSubscribleChannel") {
					//ws管理设置
					wsLoginMng.setWs(ws, jsonMsg.userID);
					// added by yu.sun for 权限设置，防止页面刷新时，未将可编辑状态设置为false 2019-07-12
				} else if (resquestType === 'addwsAuzRelate') {
					wsAuthortionMap.set(ws, "edit")
				} else if (resquestType === 'delwsAuzRelate') {
					wsAuthortionMap.delete(ws);
				}
				// end
				else if (resquestType !== "Login" && resquestType !== "manageruser" && resquestType !== "Logoff") {
					resquestType = "dowork";
					// 调用router功能，向.node发送相应参数以及回调函数
					urlRouterObj.handleFunctionWs(resquestType, ws, msg);
				}
				else {
					// 调用router功能，向.node发送相应参数以及回调函数
					urlRouterObj.handleFunctionWs(resquestType, ws, msg);
				}
			}
		}
	}

	// 接收客户端发送来的请求
	ws.on('message', function (msg) {
		// console.log("ws: " + msg);
		// 处理由客户端发来的数据
		dataServerReceiveMsgFunc(false, ws, msg);
	});
	ws.on('close', function () {
		if (wsLoginMng.hasWs(ws)) {
			var userID = wsLoginMng.getWs(ws);
			if (userID !== undefined) {
				clearModelOperateTimestampInfoAtBrowserRefreshOrClose(userID);
			}
			if (wsLoginMng.checkWs(ws, userID)) {
				var requestType = 'Logoff';
				var the_query = {
					requestType: requestType,
					type: '',
					userID: userID
				}
				console.log(the_query);
				// 当前正在部署的用户执行了刷新，断网等
				console.log(global.deployingExist)
				if (global.deployingExist !== undefined && the_query.userID === global.deployingExist) {
					global.deployExsit = true;
					global.deployingExist = undefined;
					console.log(global.deployingExist)
				}
				urlRouterObj.handleFunctionInnerRequest(requestType, JSON.stringify(the_query));
			}
			//删除ws信息
			wsLoginMng.deleteWs(ws);
		}
		// added by yu.sun for 权限设置，防止页面刷新时，未将可编辑状态设置为false 2019-07-12
		if (wsAuthortionMap.has(ws)) {
			checkStatusReset();
			wsAuthortionMap.delete(ws);
		}
		// end
	})
}

// 订阅回调函数
function subscribCallback(msg) {
	//订阅数据回推增加标志，区分是订阅数据返回还是请求结构返回
	var dataMessage = JSON.parse(msg);
	if (typeof dataMessage == 'object' && dataMessage) {
		dataMessage.dataType = 1;
		var userID = dataMessage.userID;
		msg = JSON.stringify(dataMessage);
		var specifiedWs = wsLoginMng.getSpecifiedWs(userID);
		// console.log(specifiedWs);
		if (specifiedWs && specifiedWs.readyState && specifiedWs.readyState === 1) {
			specifiedWs.send(msg);
		}
	}
}


/**
 * @class        clearModelOperateTimestampInfoAtServerStart
 * @description  服务启动时，清空页面、模型的操作信息
 * @param        {undefined}        
 * @return       {undefined}
 */

function clearModelOperateTimestampInfoAtServerStart() {
	// 先获取Projects\ProjectManager.json中的default即当前工程id
	var projectDeployTxt = fs.readFileSync( './Projects/'+ 'ProjectManager.json', 'utf-8');
	var projectDeployPath = eval("(" + projectDeployTxt + ")");
	var dataModelConfigPath = './Projects/' + 'Project-' + projectDeployPath.default + '/config/' + 'GraphicModelManager.json';
	var pageConfigPath = './Projects/' + 'Project-' + projectDeployPath.default + '/config/' + 'PageConfig.json';
	// 清空所有操作模型的信息
	fileOperateObj.exists(dataModelConfigPath, function (exists) {
		if(exists){
			var modelOperateinfo = fileOperateObj.syncReadFile(dataModelConfigPath, "utf-8");
			if (typeof modelOperateinfo === 'string') {
				modelOperateinfo = JSON.parse(modelOperateinfo);
			}
			for (var libraryName in modelOperateinfo) {
				if (libraryName === IDS_MODE_BASIC_TIMPSTAPE || libraryName === IDS_MODE_COMPOSITE_TIMPSTAPE || libraryName === IDS_MODE_ASSOCIATED_TIMPSTAPE) {
					modelOperateinfo[libraryName] = {};
				}
			}
			if (typeof modelOperateinfo === 'object') {
				modelOperateinfo = JSON.stringify(modelOperateinfo)
			}
			// fileOperateObj.asynWriteFile(1, dataModelConfigPath, modelOperateinfo, 'utf-8', function (err) {
			// 	console.log(err);
			// })
			fileOperateObj.syncWriteFile(1, dataModelConfigPath, modelOperateinfo, 'utf-8');
		}
	});

	// 清空所有操作页面的信息
	fileOperateObj.exists(pageConfigPath, function (exists) {
		if(exists){
			var configPageInfo = fileOperateObj.syncReadFile(pageConfigPath, "utf-8");
			// modified by yu.sun for 功能模块页面组权限配置 2019-07-15
			if (typeof configPageInfo === 'string') {
				if (configPageInfo.indexOf("pageGroupMng") === -1) {
					// 加密状态
					try {
						configPageInfo = KMEncryption.decrypt(configPageInfo, "pageConfig");
					} catch(err) {
						configPageInfo = configPageInfo.substring(1, configPageInfo.length - 1);
						configPageInfo = KMEncryption.decrypt(configPageInfo, "pageConfig");
					}
				}

				configPageInfo = JSON.parse(configPageInfo);
			}
			for (var pageName in configPageInfo) {
				if (pageName === 'multiUnserEditPage') {
					configPageInfo[pageName] = {};
				}
			}
			if (typeof configPageInfo === 'object') {
				configPageInfo = JSON.stringify(configPageInfo)
			}
			// fileOperateObj.asynWriteFile(1, pageConfigPath, configPageInfo, 'utf-8', function (err) {
			// 	console.log(err);
			// })
			configPageInfo = KMEncryption.encrypt(configPageInfo, "pageConfig");
			fileOperateObj.syncWriteFile(1, pageConfigPath, JSON.stringify(configPageInfo), 'utf-8');
			// end
		}
	})
}


/**
 * @class        clearModelOperateTimestampInfoAtBrowserRefreshOrClose
 * @description  浏览器刷新、关闭时，清空页面、模型的操作信息
 * @param        {string}         当前用户UserHandle
 * @return       {undefined}
 */

function clearModelOperateTimestampInfoAtBrowserRefreshOrClose(userhandle) {
	// 先获取Projects\ProjectManager.json中的default即当前工程id
	var projectDeployTxt = fs.readFileSync('./Projects/' + 'ProjectManager.json', 'utf-8');
	var projectDeployPath = eval("(" + projectDeployTxt + ")");
	var dataModelConfigPath = './Projects/' + 'Project-' + projectDeployPath.default + '/config/' + 'GraphicModelManager.json';
	var pageConfigPath = './Projects/' + 'Project-' + projectDeployPath.default + '/config/' + 'PageConfig.json';
	// 清空当前用户操作模型的信息
	fileOperateObj.exists(dataModelConfigPath, function (exists) {
		if (exists) {
			var configModelobj = fileOperateObj.syncReadFile(dataModelConfigPath, "utf-8");
			if (typeof configModelobj === 'string') {
				configModelobj = JSON.parse(configModelobj);
			}
			for (var libraryName in configModelobj) {
				if (libraryName === IDS_MODE_BASIC_TIMPSTAPE || libraryName === IDS_MODE_COMPOSITE_TIMPSTAPE || libraryName === IDS_MODE_ASSOCIATED_TIMPSTAPE) {
					for (var modelName in configModelobj[libraryName]) {
						for (var modelinfo in configModelobj[libraryName][modelName]) {
							for (var modelUserIndex = 0; modelUserIndex < configModelobj[libraryName][modelName][modelinfo].length; modelUserIndex++) {
								if (configModelobj[libraryName][modelName][modelinfo][modelUserIndex].modelUser === userhandle) {
									configModelobj[libraryName][modelName][modelinfo].splice(modelUserIndex, 1);
									if (configModelobj[libraryName][modelName][modelinfo].length === 0) {
										// 该模型下无用户操作信息 则彻底删掉该模型信息
										delete configModelobj[libraryName][modelName];
										break;
									}
								}
							}
							break;
						}
					}
				}
			};
			// 反序列化 将配置文件写入后台
			if (typeof configModelobj === 'object') {
				configModelobj = JSON.stringify(configModelobj)
			}
			// fileOperateObj.asynWriteFile(1, dataModelConfigPath, configModelobj, 'utf-8', function (err) {
			// 	// console.log(err);
			// })
			fileOperateObj.syncWriteFile(1, dataModelConfigPath, configModelobj, 'utf-8');
		}
	});

	// 清空当前用户操作页面的信息
	fileOperateObj.exists(pageConfigPath, function (exists) {
		if (exists) {
			var pageConfigObj = fileOperateObj.syncReadFile(pageConfigPath, "utf-8");
			if (typeof pageConfigObj === 'string') {
				if (pageConfigObj.indexOf("pageGroupMng") === -1) {
					// 加密状态
					try {
						pageConfigObj = KMEncryption.decrypt(pageConfigObj, "pageConfig");
					} catch(err) {
						pageConfigObj = pageConfigObj.substring(1, pageConfigObj.length - 1);
						pageConfigObj = KMEncryption.decrypt(pageConfigObj, "pageConfig");
					}
					
				}
				pageConfigObj = JSON.parse(pageConfigObj);
			}
			 for(var pageName in pageConfigObj.multiUnserEditPage){
				for(var listUser in pageConfigObj.multiUnserEditPage[pageName]){
					for(var userIndex=0;userIndex<pageConfigObj.multiUnserEditPage[pageName][listUser].length;userIndex++){
						if(pageConfigObj.multiUnserEditPage[pageName][listUser][userIndex].userHandle === userhandle){
							pageConfigObj.multiUnserEditPage[pageName][listUser].splice(userIndex, 1);
							if (pageConfigObj.multiUnserEditPage[pageName][listUser].length === 0) {
								// 该模型下无用户操作信息 则彻底删掉该模型信息
								delete pageConfigObj.multiUnserEditPage[pageName];
								break; 
							}
						}
					}
				}
			 }
			 if (typeof pageConfigObj === 'object') {
				pageConfigObj = JSON.stringify(pageConfigObj)
			}
			// fileOperateObj.asynWriteFile(1, pageConfigPath, pageConfigObj, 'utf-8', function (err) {
			// 	// console.log(err);
			// })
			pageConfigObj = KMEncryption.encrypt(pageConfigObj, "pageConfig");
			fileOperateObj.syncWriteFile(1, pageConfigPath, JSON.stringify(pageConfigObj), 'utf-8');
		}
	})
}


var serverMng = new Server();
var createFileServerRes = serverMng.createFileServer(serverMng.config.http_type, serverMng.config.innerNet.host_ip + ":" + serverMng.config.innerNet.file_listen_port, fileServerHandleFunc);
var createDataServerRes = serverMng.createDataServer(serverMng.config.ws_type, kmCSTypeEnum.KM_COMM_SERVER, serverMng.config.innerNet.host_ip + ":" + serverMng.config.innerNet.data_listen_port, dataServerHandleFunc);
var dataServer = serverMng.getDataServer();
serverMng.registerSubscribFunc(true, subscribCallback);
// 根据文件服务创建结果输出相关信息
if (createFileServerRes) {
	var fileServerType = undefined;
	if (serverMng.config.http_type === kmProtocolTypeEnum.KM_COMM_HTTP) {
		fileServerType = 'Http';
	} else if (serverMng.config.http_type === kmProtocolTypeEnum.KM_COMM_HTTPS) {
		fileServerType = 'Https';
	}
	if (fileServerType !== undefined) {
		console.log(fileServerType + " server is listening on port:" + serverMng.config.innerNet.file_listen_port);
	}
} else {
	console.log("fileServer create failure !");
}
// 根据数据服务创建结果输出相关信息
if (createDataServerRes) {
	var dataServerType = undefined;
	if (serverMng.config.ws_type === kmProtocolTypeEnum.KM_COMM_WS) {
		dataServerType = 'Ws';
	} else if (serverMng.config.ws_type === kmProtocolTypeEnum.KM_COMM_WSS) {
		dataServerType = 'Wss';
	}
	if (dataServerType !== undefined) {
		console.log(dataServerType + " server is listening on port:" + serverMng.config.innerNet.data_listen_port);
	}
} else {
	console.log("dataServer create failure !");
}
clearModelOperateTimestampInfoAtServerStart();



// var schedule = require('node-schedule');
// function scheduleCronstyle() {
// 	var date = new Date(2018, 10, 20, 00, 00, 00);
// 	schedule.scheduleJob(date, function () {
// 		serverMng.close();
// 		console.log('scheduleCronstyle:' + new Date());
// 	});
// 	var curDate = new Date();
// 	if (curDate > date) {
// 		serverMng.close();
// 		console.log('scheduleCronstyle:' + new Date());
// 	}

// }
// scheduleCronstyle();
function addIconClassToFile() {
	var projectConfigPath = '../../DevelopServer/Server/Projects/ProjectManager.json';
	fileOperateObj.exists(projectConfigPath, function (exists) {
		if (exists) {
			// 此段代码mini时不识别，故修改
			var content = fileOperateObj.syncReadFile(projectConfigPath, "utf-8");
			if (typeof content === "string") {
				content = JSON.parse(content)
			}
			var currentProName = 'Project-' + content["default"];
			// 判断是否有文件夹及文件存在
			var filePath = "./Projects/" + currentProName + "/res/iconimages"
			fileOperateObj.exists(filePath, function (exists) {
				if (exists) {
                    createEasyuiIconJson (currentProName)
				} else {
                    fs.mkdir(filePath, function(err) {
                        if(err) {
				console.log("创建文件夹失败：" + err);
			} else {
		                createEasyuiIconJson (currentProName)
			}
		    })
		}
	})
			
		} else {
			console.log(projectConfigPath + ' :does not exist!');
		}
	});
}
function createEasyuiIconJson (currentProName) {
	var iconJsonPath = "./Projects/" + currentProName + "/res/imageResource/icon-name.json"
	fileOperateObj.exists(iconJsonPath, function (exists) {
		if(exists) {
			var cssPath = '../../ThirdParty/jquery-easyui/themes/icon.css'
			fileOperateObj.asynReadFile(cssPath, 'utf-8', function (err, data) {
				if (err){
				   console.log(err);
				};
				var iconCss = data;
				var iconConfig = require('../../DevelopServer/Server/Projects/' + currentProName + '/res/imageResource/icon-name.json').UIPICTURE;
				for (var i = 0; i < iconConfig.length; i++) {
					var strAdd = "." + iconConfig[i].class + "{\n background:url('../../../DevelopServer/Server/Projects/"  + currentProName + '/res/iconimages/' + iconConfig[i].name + "') no-repeat center center;\n}\n";
					var iconCss = iconCss + strAdd;
				}
				fileOperateObj.syncWriteFile(1, cssPath, iconCss, 'utf-8');
			});
		} else {
			var json = {"UIPICTURE":[]};
			fs.writeFile(iconJsonPath, JSON.stringify(json), 'utf-8', function(err) {
				if(err) {
					console.log("创建文件失败:" + err);
				} 
			})
		}
	})
}
addIconClassToFile();

// added by yu.sun 创建projectAuz.json
function createPageGroupAuz() {
	var projectConfigPath = '../../DevelopServer/Server/Projects/ProjectManager.json';
	fileOperateObj.exists(projectConfigPath, function (exists) {
		if (exists) {
			// 此段代码mini时不识别，故修改
			var content = fileOperateObj.syncReadFile(projectConfigPath, "utf-8");
			if (typeof content === "string") {
				content = JSON.parse(content)
			}
			var currentProName = 'Project-' + content["default"];
			var filePath = "./Projects/" + currentProName + "/Config/" + "ProjectAuz.json"
			fileOperateObj.exists(filePath, function (exists) {
				if (!exists) {
					var file = {};
					file.pageGroupAuz = new Array();
					file.key = new Array();
					fileOperateObj.syncWriteFile(1, filePath, JSON.stringify(file), 'utf-8');
				}
			})
		}
	})
}
createPageGroupAuz();
// end
// added by yu.sun pageconfig里面的
function checkStatusReset() {
	var projectConfigPath = '../../DevelopServer/Server/Projects/ProjectManager.json';
	fileOperateObj.exists(projectConfigPath, function (exists) {
		if (exists) {
			// 此段代码mini时不识别，故修改
			var content = fileOperateObj.syncReadFile(projectConfigPath, "utf-8");
			if (typeof content === "string") {
				content = JSON.parse(content)
			}
			var prjectPath = 'Project-' + content["default"];

			var filePath = "./Projects/" + prjectPath + "/Config/" + "PageConfig.json"
			fileOperateObj.exists(filePath, function (exists) {
				if (exists) {
					fileOperateObj.asynReadFile(filePath, 'utf-8', function (err, data) {
						if (err) {
							console.log(err);
						} else {
							// 判断当前pageConfig是否处于加密状态
							if (data.indexOf("pageGroupMng") === -1) {
								// 加密状态
								try {
									data = KMEncryption.decrypt(data, "pageConfig");
								} catch(err) {
									data = data.substring(1, data.length - 1);
									data = KMEncryption.decrypt(data, "pageConfig");
								}
								
							}
							data = JSON.parse(data);
							data.authorityStatus = false;
							data = KMEncryption.encrypt(JSON.stringify(data), "pageConfig");
							fileOperateObj.syncWriteFile(1, filePath, JSON.stringify(data), 'utf-8');
						}
					})
				}
			})
		}
	})

}
checkStatusReset();
// end
